/**	This file computes the critical value tables (Table 1-22) in GKM "A more powerfull subvector Anderson Rubin test in linear IV regression"
	Takes about 18 minutes to run on single processor.
**/

#include <oxstd.oxh>
#include <oxfloat.oxh>
#include <oxdraw.oxh>
#include <quadpack.oxh>
#import <solvenle>
#include <conddens.ox>

steps(const start, const dmax, const cmaxsteps)
{
	decl stepsize = 0.1;
	for(decl s = 0; s < 10; ++s){						  // increase space between quantiles if necessary in order to fit in the table
		if(dmax/stepsize<cmaxsteps-1)
			break;
		stepsize += 0.1;
	}

	decl vgrid = range(start, dmax, stepsize);
	if(!isfeq(vgrid[sizec(vgrid)-1],trunc(dmax*10)/10))	  // get to within 1 decimal from maximum (because quantile function flattens out in the end) 
		vgrid ~= trunc(dmax*10)/10;
	return vgrid;
}

main()
{
	decl time = timer();
	decl v_qlevel = <0.9,0.95,0.99>;					// quantile level = 1 - significance level
	decl vgrid, quantiles, vkap1hat;
	fuzziness(1e-2);									
	decl cmaxsteps = 80;
	for(decl k = 2; k <= 21; ++k){
		for(decl i = 0; i < sizerc(v_qlevel); ++i){
			g_dquant = v_qlevel[i];
			g_k = k;
			decl dqchi2 = quanchi(v_qlevel[i],k-1);		// quantile of chi2
			vgrid = range(0.1,300,.1);
			quantiles = vkap1hat = <>;						
			decl vqgrid;
			decl q;		
			for(decl j = 0; j < sizerc(vgrid); ++j)
			{
				g_kap1hat = vgrid[j];
				decl q = min(dqchi2,g_dquant*g_kap1hat);		// initialize the quantile
				decl ires = SolveNLE(distfunc, &q, /*iMode*/-1, /*funcJac*/<>, /*dEps1*/-1, /*dEps2*/-1,/*mxIter*/-1, /*iPrint*/0);
				q = ceil(10*q)/10;								// round upwards to 1 decimal
				if(sizec(quantiles)==1)
					vqgrid = steps(quantiles[0], dqchi2, cmaxsteps);
				if(isfeq(q, g_kap1hat))							// skip all entries where q(x) = x, where x = kap1hat
					continue;
				if(sizec(quantiles)>1 && !any(isdotfeq(q,vqgrid)))		// skip all entries where q(x_i) is not in the grid of quantiles
					continue;
				if(sizec(quantiles) && isfeq(q,quantiles[sizec(quantiles)-1]))		// also skip all entries where q(x_i) = q(x_i-1)
					continue;
				if(q > dqchi2)									// if the rounded q exceeds the (unrounded) chi2, stop
					break;								
				quantiles ~= q;
				vkap1hat ~= vgrid[j];
				if(isfeq(q,round(10*dqchi2)/10))				// stop when the rounded q is within one decimal from chi2
					break;								
			}
			g_kap1hat = 1000;									// add one final case to approximate chi2 quantile
			q = dqchi2;											// initialize at last chi2 quantile
			SolveNLE(distfunc, &q, /*iMode*/-1, /*funcJac*/<>, /*dEps1*/-1, /*dEps2*/-1,/*mxIter*/-1, /*iPrint*/0);
//			vkap1hat ~=	g_kap1hat;
			vkap1hat = 0~vkap1hat~g_kap1hat~.Inf;  				// append zero and infinite quantiles
			quantiles = 0~quantiles~q~dqchi2;
			savemat(sprint("CVtables\\k_",k,"_level_",v_qlevel[i],".in7"),vkap1hat'~quantiles',{"x","q"});
			println("Done with k = ", k, ", level = ", v_qlevel[i], " with # of steps = ", sizerc(vkap1hat));
			println("\nlapsed time: ", timespan(time),"\n");
//			println(vkap1hat'~quantiles');
		}
	}
}