/**	This file reproduces Figure 1 in GKM "A more powerfull subvector Anderson Rubin test in linear IV regression"
	Takes about half a minute to run on single processor -- most time taken by case k=2
**/

#include <oxstd.oxh>
#include <oxfloat.oxh>
#include <oxdraw.oxh>
#include <quadpack.oxh>
#include <conddens.ox>
#import <solvenle>

main()
{
	decl time = timer();
	g_dquant = 0.95;							// 1 - significance level
	decl vk = <2;5;10;20>;
	SetDrawWindow("Figure 1");
	SetDraw(SET_LINE, 2, 0, 20);
	SetDraw(SET_LINE, 3, 2, 20);
	for(decl ik = 0; ik < sizerc(vk); ++ik)
	{
		g_k = vk[ik];
		decl vgrid, quantiles, vkap1hat;

		decl dqchi2 = quanchi(g_dquant,g_k-1);					// quantile of chi2
		decl start = g_k <= 5 ? .1 : g_k==10 ? .2 : .5;			// starting point of x-axis -- adjust so that shape is similar
		decl end = g_k <= 5 ? 200 : 500;						// end point of x-axis
		vgrid = exp(range(log(start),log(end),(log(end)-log(start))/100));
		quantiles = vkap1hat = <>;						
		for(decl j = 0; j < sizerc(vgrid); ++j)
		{
			g_kap1hat = vgrid[j];
			decl q = min(dqchi2,g_dquant*g_kap1hat);		// initialize the quantile
			decl ires = SolveNLE(distfunc, &q, /*iMode*/-1, /*funcJac*/<>, /*dEps1*/-1, /*dEps2*/-1,/*mxIter*/-1, /*iPrint*/0);
			quantiles ~= q;
			vkap1hat ~= vgrid[j];
			fuzziness(dqchi2 < 10 ? 1e-2 : 1e-1);							// set fuzziness to compare cond quantile to chi2 to 1 decimal (if cond quantile is within 1 decimal of asymptotic, stop)
			if(isfeq(q,dqchi2))
				break;
		}			 
		DrawTitle(ik, sprint("k = ", g_k));
		DrawXMatrix(ik, quantiles|dqchi2, {"conditional","$\\chi^2(k-1)$"}, log(vkap1hat), "");
		DrawAxisAuto(ik, 1);
		DrawAdjust(ADJ_AXISSCALE, AXIS_LOG);
		DrawAdjust(ADJ_AREAMATRIX, TRUE, FALSE, FALSE, 800);
		DrawLegend(ik,0,0,/*Hidden*/TRUE);
		DrawText(ik, "$\\hat{\\kappa}_1$", -1, -1, -1, -1, TEXT_XLABEL);
	}
	ShowDrawWindow();
	SaveDrawWindow("conditional_quantile_function.eps");			// uncomment if you wish to save

	println("\nlapsed time: ", timespan(time),"\n");
}