/**	This file reproduces Figure 3 in GKM "A more powerfull subvector Anderson Rubin test in linear IV regression"
	Requires as input Powerbounds.in7 (produced by powerbounds.ox) and PowerCondTest.in7 (produced by PowerCondTest.ox).
**/

#include <oxstd.oxh>
#include <oxdraw.oxh>
#import "Database"

main()
{
	decl time = timer();
	decl alpha = 0.05;					// significance level of test
		
	decl Varnames = {"Conditional", "GKMC"};
	decl bounds = new Database();
	bounds.Load("Powerbounds.in7");
	decl power = new Database();
	power.Load("PowerCondTest.in7");
	decl k = 5;
	decl vkappa2;
	decl vkap1mkap2 = <>;							// kappa_1-kappa_2 under alternative
	decl id = -1;									// initialize indix of kappa_1-kappa_2
	decl mPower2D, mPower3D;
	decl asPower = {"Power bound", "$\\varphi_c$", "$\\varphi_{GKMC}$"};		// names for the 2-D plot
	SetDrawWindow("Figure 3");
	SetDraw(SET_LINE, 2, 2, 30);
	SetDraw(SET_LINE, 1, 0, 30);
	SetDraw(SET_LINE, 3, 1, 30);
	for(decl i=0; i<bounds.GetSize(); ++i)
	{
		if(k != bounds.GetVar("k")[i])
			continue;
		decl kap1mkap2 = bounds.GetVar("kappa1-kappa2")[i];	
		if(vkap1mkap2 == <> || kap1mkap2 != vkap1mkap2[id]){
			if(id==0){								// plot 2D power curves in the same figure for kap1mkap2=0
				DrawTitle(1, "Power curves when $\\kappa_1=\\kappa_2$");
				DrawXMatrix(1, mPower2D, asPower, vkappa2, "", 0, 1);
				DrawText(1, "$\\kappa_2$", -1, -1,-1, -1, TEXT_XLABEL); 
				DrawAdjust(ADJ_AREAMATRIX, -1,-1, 0, 760);
			}				
			vkappa2 = bounds.GetVar("kappa2")[i];		// restart vkappa2
			mPower2D = bounds.GetVar("ALFD")[i]
					| power.GetVar(Varnames)[i][]';	 	// restart power
			vkap1mkap2 ~= kap1mkap2;
			++id;
		}
		else{
			vkappa2 ~= bounds.GetVar("kappa2")[i];
			mPower2D ~= (bounds.GetVar("ALFD")[i]
					| power.GetVar(Varnames)[i][]');	 	// restart power
		}
	}
	decl ckap1mkap2 = sizerc(vkap1mkap2), ckappa2 = sizerc(vkappa2);
	mPower3D = zeros(ckap1mkap2, ckappa2);
	for(decl i=0; i<bounds.GetSize(); ++i)
	{
		if(k != bounds.GetVar("k")[i])
			continue;
		decl kap1mkap2 = bounds.GetVar("kappa1-kappa2")[i];
		decl kappa2 = bounds.GetVar("kappa2")[i];
		decl arg = bounds.GetVar({"ALFD"})[i] ~ power.GetVar({"Conditional"})[i];
		mPower3D[vecindex(vkap1mkap2 .== kap1mkap2)][vecindex(vkappa2 .== kappa2)] = arg*<-1;1>;	// Power of conditional test - EMW power bound
	}
	DrawTitle(0, "Power of $\\varphi_c$ minus power bound");
	DrawXYZ(0, vkappa2, vkap1mkap2, mPower3D', 0, "$\\kappa_2$", "$\\kappa_1-\\kappa_2$", "");
	DrawText(0, "$\\kappa_2$", -1, -1,-1, -1, TEXT_XLABEL); 
	DrawText(0, "$\\kappa_1-\\kappa_2$", -1, -1,-1, -1, TEXT_YLABEL); 
	DrawText(1, "$\\kappa_2$", -1, -1,-1, -1, TEXT_XLABEL); 
	DrawLegend(1, 500, 500, FALSE); 
	DrawAdjust(ADJ_AREA_Y, 1, 0, 1);
	DrawAdjust(ADJ_LEGEND, 1, 1, -1, -1, 0);
	DrawAdjust(ADJ_PAPERSCALE, 50);
	ShowDrawWindow();
	SaveDrawWindow(sprint("PowerComparisons.eps"));
	delete bounds;
	delete power;

}