/**	This file replicates Figure 4 in GKM "A more powerfull subvector Anderson Rubin test in linear IV regression"
	It requires file DensityWishartEigvals.dll, and must be run on 64 bit Windows using OxMetrics version 7 or 7.1
	It also requires CVtables\k_5_level_0.95.in7 (produced by Critical_Value_Tables.ox), Powerbounds.in7 (produced by powerbounds.ox) and PowerCondTest.in7 (produced by PowerCondTest.ox).
**/

#include <oxstd.oxh>
#include <oxfloat.oxh>
#include <oxdraw.oxh>
#include <oxprob.oxh>
#include <quadpack.oxh>		
#import "Database"
#include <interpolate.ox>		

extern "DensityWishartEigvals,mhg_manyvals" density(const MAX, const alpha, const b, const X, const Y);

decl g_M = 200;
decl g_k, g_kap2hatbound, g_quants;

jointdensEvals(const k, const kappa, const kaphat)
{
/** Computes the value of the joint density of the eigenvalues at kappa,kaphat
	in: k degrees of freedom of Wishart
		kappa vector of eigenvalues of NC matrix
		kaphat vector of eigenvalues of Wishart matrix
	returns density
**/																						 
	decl a1 = sqrt(M_PI)/2^k/gammafact(k/2)/gammafact((k-1)/2);
	decl a2 = prodc(vec(kaphat))^((k-3)/2) * (kaphat[0]-kaphat[1]); 			
	decl a3 = exp(-sumc(vec(kaphat))/2)*density(g_M,/*alpha*/2, k/2, kappa/4, kaphat);
	return a1*a2*a3; 
}

class NRP
{
	decl m_kappa, m_kap1hat, m_fchi2;		//fchi2 is a flag: if true, use chi2 cvs, otherwise conditional
	NRP(const fChi2);
	Setkappa1(const kappa1);
	toIntwrt2(const x2);
	toIntwrt1(const x1);
};
NRP::NRP(const fChi2)
{
	m_fchi2 = fChi2;
}
NRP::Setkappa1(const kappa1)
{
	m_kappa = kappa1|0;
}
NRP::toIntwrt2(const x2)
{
	decl pdf = jointdensEvals(g_k, m_kappa, m_kap1hat~x2);
	return pdf;// > 0 ? pdf : 0;
}
NRP::toIntwrt1(const x1)
{
	decl result, sterr;
	decl kap2hatbound;
	if(m_fchi2)
		kap2hatbound = min(g_kap2hatbound,x1);	// use constant cv (if less than x1)
	else{
		decl idx = sumc(g_quants[][0].<x1);		// index of ceiling in x values.
		decl x_lo, x_up;
		x_lo = g_quants[idx-1][0];
		x_up = g_quants[idx][0];
		decl y_up =  g_quants[idx][1], y_lo = g_quants[idx-1][1];
		kap2hatbound = interpolate(x1, x_up, x_lo, y_up, y_lo);	   // conditional critical value interpolated from table
	}
	m_kap1hat = x1;
	decl iq = QAG(toIntwrt2, 0, kap2hatbound, 1, &result, &sterr);
	if(iq){
		println("%c",{"kappa1","x1","c(x1)","int","sterr","q"},m_kappa[0]~x1~kap2hatbound~result~sterr~iq);
		iq = QAG(toIntwrt2, 0, kap2hatbound, 2, &result, &sterr);
		if(iq){
			iq = QAG(toIntwrt2, 0, kap2hatbound, 3, &result, &sterr);
			if(iq){
				iq = QAG(toIntwrt2, 0, kap2hatbound, 4, &result, &sterr);
				if(iq){
					iq = QAG(toIntwrt2, 0, kap2hatbound, 5, &result, &sterr);
					if(iq)
						iq = QAG(toIntwrt2, 0, kap2hatbound, 6, &result, &sterr);
				}
			}
		}
	}
	
	return result; 			
}

main()									
{

	decl time = timer();
	decl level = 0.05;
	g_k = 5;				
	decl ckappa1 = 42; 
	decl lb=0, ub=log(101), stepsize=ub/(ckappa1-1); 		//lb, ub, stepsize: lower and upper bound and stepsize for grid on kappa1.
	decl vkappa1 = exp(range(lb, ub, stepsize))-1;			//Set grid for kappa1 under H0.
	// check density is a density, i.e., integrates to 1
	decl check = new NRP(TRUE), rescheck, sterr;
	g_kap2hatbound = .Inf; check.Setkappa1(0);
	QPEPS(0, 1e-5);				// sets desired accuracy
	QAGI(check.toIntwrt1, 0.0001, 1, &rescheck, &sterr);
	println("Density integrates to ", rescheck);
	delete check;
	fuzziness(1e-5);
	if(!isfeq(rescheck, 1)){
		println("Density is wrong, check constant of integration");
		return;
	}
	SetDrawWindow("Figure 4");
	SetDraw(SET_LINE, 4, 1, 30);
	SetDraw(SET_LINE, 2, 0, 30);
	SetDraw(SET_LINE, 3, 2, 30);
	SetDraw(SET_LINE, 5, 3, 30);
	SetDraw(SET_LINE, 1, 0, 30);
	// first, drow size
	decl mNullRejFreq = vkappa1'~ constant(.NaN,ckappa1,3);	 				// stores NRP of cond AR test at different values of kappa1 
	g_quants = loadmat(sprint("CVtables\\k_",g_k,"_level_",1-level,".in7"));   			// cv table
	g_kap2hatbound = quanchi(1-level, g_k-1);
	DrawTitle(0, sprint("NRP, $k=",g_k, "$, $m_W=1$, $\\alpha = $",level,"$"));	  
	DrawXMatrix(0, constant(level,1,ckappa1),"",vkappa1,"", 0, 4);
	decl size = loadmat("condsize.csv");
	if(isfeq(size[][0], vkappa1'))					// if you've stored already NRP of cond and GKMC from Figure 2
		mNullRejFreq[][0:1] = size[][1:2];			// retrieve NRPs from file	
	else{
		for(decl fchi2 = 0; fchi2 < 2; ++fchi2){
			decl model = new NRP(fchi2);
			parallel for(decl i = 0; i < ckappa1; ++i){
				decl modeli = clone(model);
				modeli.Setkappa1(vkappa1[i]);	
				decl result, sterr;
//				decl iq = QAG(modeli.toIntwrt1, 0.0001, 200, 6, &result, &sterr);		// takes longer, but can be more accurate in rare cases
				decl iq = QAGI(modeli.toIntwrt1, 0.0001, 1, &result, &sterr);
				mNullRejFreq[i][fchi2] = (1-result);
			}
			delete model;
			println("\nlapsed time: ", timespan(time),"\n");
		}
	}
	// now compute NRP of test with adjusted cvs
	decl model = new NRP(FALSE);
	g_quants[:8][1] /= 5;						 // divide first 8 (excluding the first one which is zero) conditional cvs by 5.
	parallel for(decl i = 0; i < ckappa1; ++i){
		decl modeli = clone(model);
		modeli.Setkappa1(vkappa1[i]);	
		decl result, sterr;
//		QAG(modeli.toIntwrt1, 0.0001, 200, 6, &result, &sterr);		// takes longer, but can be more accurate in rare cases
		QAGI(modeli.toIntwrt1, 0.0001, 1, &result, &sterr);
		mNullRejFreq[i][2] = 1-result;
	}
	delete model;
	println("\nlapsed time: ", timespan(time),"\n");
	
	DrawXMatrix(0, mNullRejFreq',{"$\\varphi_c$","$\\varphi_{GKMC}$","$\\varphi_{adj}$"},vkappa1,"", 0, <2;3;5>);
	DrawText(0, "$\\kappa_1$", -1, -1, -1, -1, TEXT_XLABEL);
	DrawLegend(0, /*iOffsX*/500, /*iOffsY*/500, FALSE);		
	DrawAdjust(ADJ_LEGEND, 0, 1, -1, -1, 0);
	DrawAdjust(ADJ_AREAMATRIX, TRUE, FALSE, FALSE, 800);

	// Next, draw power curves
	decl bounds = new Database();
	bounds.Load("Powerbounds.in7");
	decl power = new Database();
	power.Load("PowerCondTest.in7");
	decl Varnames = {"Conditional", "GKMC"};
	decl vkappa2;
	decl id = 0;									
	decl mPower2D;
	decl kap1mkap2 = 0;								// specific value under H1 of kappa_1-kappa_2
	decl asPower = {"power bound", "$\\varphi_c$", "$\\varphi_{GKMC}$", "$\\varphi_{adj}$"};		// names for the 2-D plot
	for(decl i=0; i<bounds.GetSize(); ++i)
	{
		if(g_k != bounds.GetVar("k")[i])
			continue;
		if(bounds.GetVar("kappa1-kappa2")[i] == kap1mkap2){	
			if(id == 0){
				vkappa2 = bounds.GetVar("kappa2")[i];		
				mPower2D = bounds.GetVar("ALFD")[i]
						| power.GetVar(Varnames)[i][]';	 	
				++id;
			}
			else{
				if(bounds.GetVar("ALFD")[i]>.3)			// stop half way, to zoom in on differences
					continue;
				vkappa2 ~= bounds.GetVar("kappa2")[i];
				mPower2D ~= (bounds.GetVar("ALFD")[i]
						| power.GetVar(Varnames)[i][]');	 	
			}
		}
	}
	// draw power of test with adjusted cvs
	decl crepalt = 1e5;
	ranseed(11111111);										// same as for Figure 3
	mPower2D |= 0;											// append a new row to be filled with power of adjusted test
	parallel for(decl i = 0; i < sizerc(vkappa2); ++i){
		decl kappa2 = vkappa2[i];
	
		//Draw values under the alternative to calculate power:
		decl kappa1 = kap1mkap2+kappa2;				//largest eigenvalue
		decl mkappahat = new matrix[crepalt][2];
	
		for(decl i=0; i < crepalt; ++i)
		{
			decl mu = rann(g_k,2);
			decl mnc = zeros(g_k,2);				// initialize noncentrality (mean) of normals
			mnc[0][0] = sqrt(kappa1);			
			mnc[1][1] = sqrt(kappa2);			
			decl mX = mu+mnc;						// draw from non-central Wishart (via Normals)
			decl kaphat;
			eigensym(mX'mX, &kaphat);				// obtain its eigenvalues
			mkappahat[i][] = kaphat;	  	 		//store drawn eigenvalues
		}							  
		decl mtestfunc = new matrix[crepalt][1];	// value of conditional and GKMC test functions for each draw (Nx2 matrix of 0s and 1s)
		for(decl ix = 0; ix<crepalt; ++ix){
			decl x = mkappahat[ix][];				// largest~smallest eigenvalues
			decl idx = sumc(g_quants[][0].<x[0]);	// index of ceiling in x1 values (to be used as conditional cv)
			decl x0, x_lo, x_up;
			x0 = x[0];
			x_lo = g_quants[idx-1][0];
			x_up = g_quants[idx][0];
			decl y_up =  g_quants[idx][1], y_lo = g_quants[idx-1][1];
			decl dcv = interpolate(x0, x_up, x_lo, y_up, y_lo);	   // conditional critical value interpolated from table
			mtestfunc[ix][0] = x[1] .> dcv;			// conditional test function
		}				
		mPower2D[3][i] = meanc(mtestfunc);					   
	}	
	
	DrawTitle(1, sprint("Power curves when $\\kappa_1-\\kappa_2=$",kap1mkap2));
	DrawXMatrix(1, mPower2D, asPower, vkappa2, "", 0, <1;2;3;5>);
	DrawText(1, "$\\kappa_2$", -1, -1,-1, -1, TEXT_XLABEL); 
	DrawAdjust(ADJ_AREAMATRIX, -1,-1, 0, 760);
	DrawAdjust(ADJ_LEGEND, 1, 1, -1, -1, 0);
	DrawAdjust(ADJ_PAPERSCALE, 50);

	ShowDrawWindow();
	SaveDrawWindow(sprint("adjusted.eps"));
	println("\nlapsed time: ", timespan(time),"\n");
	
}
					 	