/**	This file replicates Figure 8 in GKM "A more powerfull subvector Anderson Rubin test in linear IV regression"
	Requires Powerbounds.in7 (produced by powerbounds.ox).
**/

#include <oxstd.oxh>
#include <oxdraw.oxh>
#import "Database"

main()
{
	decl time = timer();
	decl alpha = 0.05;								// significance level of test
	decl Varnames = {"EMW", "ALFD"};				// variables holding upper bounds
	decl bounds = new Database();
	bounds.Load("Powerbounds.in7");
	decl vk = <2;5;10;20>;
	decl vkappa2;									// kappa2 under alternative
	decl vkap1mkap2;								// kappa_1-kappa_2 under alternative
	for(decl ik = 0; ik < sizerc(vk); ++ik){
		decl k = vk[ik];			  					// which k to plot
		for(decl j = 0; j<8; ++j){		
			vkap1mkap2 = <>;							// kappa_1-kappa_2 under alternative
			decl id = -1;								// initialize indix of kap1mkap2
			decl mEpsilon3D;							// difference between power bounds in EMW algorithm step 9
			SetDraw(SET_LINE, 2, 2, 30);
			SetDraw(SET_LINE, 1, 0, 20);
			SetDraw(SET_LINE, 3, 1, 20);
			SetDraw(SET_LINE, 4, 3, 20);
			for(decl i=0; i<bounds.GetSize(); ++i)
			{
				if(k != bounds.GetVar("k")[i])
					continue;
				decl kap1mkap2 = bounds.GetVar("kappa1-kappa2")[i];	
				if(vkap1mkap2 == <> || kap1mkap2 != vkap1mkap2[id]){
					vkappa2 = bounds.GetVar("kappa2")[i];		
					vkap1mkap2 ~= kap1mkap2;
					++id;
				}
				else
					vkappa2 ~= bounds.GetVar("kappa2")[i];
			}
			mEpsilon3D = zeros(sizerc(vkap1mkap2), sizerc(vkappa2));
			for(decl i=0; i<bounds.GetSize(); ++i)
			{
				if(k != bounds.GetVar("k")[i])
					continue;
				decl kap1mkap2 = bounds.GetVar("kappa1-kappa2")[i];
				decl kappa2 = bounds.GetVar("kappa2")[i];
				decl epsilon = bounds.GetVar({"EMW", "ALFD"})[i][]*<1;-1>;
				mEpsilon3D[vecindex(vkap1mkap2 .== kap1mkap2)][vecindex(vkappa2 .== kappa2)] = epsilon;		// see step of EMW algorithm in GKM Section S.2.3.2
			}										  
			DrawTitle(ik, sprint("k = ",k));
			DrawXYZ(ik, vkappa2, vkap1mkap2, mEpsilon3D', 0, "$\\kappa_2$", "$\\kappa_1-\\kappa_2$", "");
			DrawText(ik, "$\\kappa_2$", -1, -1,-1, -1, TEXT_XLABEL); 
			DrawText(ik, "$\\kappa_1-\\kappa_2$", -1, -1,-1, -1, TEXT_YLABEL); 
		}
	}
	ShowDrawWindow();
	SaveDrawWindow(sprint("EpsilonALFD.eps"));
	delete bounds;
}