/**	This file replicates Figure 6 in GKM "A more powerfull subvector Anderson Rubin test in linear IV regression"
	It requires file DensityWishartEigvals.dll, and must be run on 64 bit Windows using OxMetrics version 7 or 7.1
	It also requires tables of conditional critical values, which should have been stored in the format: CVtables\k_5_level_0.95.in7.
	If not computed, these critical values can be obtained by running the file Critical_Value_Tables.ox
**/

#include <oxstd.oxh>
#include <oxfloat.oxh>
#include <oxdraw.oxh>
#include <oxprob.oxh>
#include <quadpack.oxh>		
#include <interpolate.ox>		

extern "DensityWishartEigvals,mhg_manyvals" density(const MAX, const alpha, const b, const X, const Y);

decl g_M = 200;
decl g_k, g_kap2hatbound, g_quants;

jointdensEvals(const k, const kappa, const kaphat)
{
/** Computes the value of the joint density of the eigenvalues at kappa,kaphat
	in: k degrees of freedom of Wishart
		kappa vector of eigenvalues of NC matrix
		kaphat vector of eigenvalues of Wishart matrix
	returns density
**/																						 
	decl a1 = sqrt(M_PI)/2^k/gammafact(k/2)/gammafact((k-1)/2);
	decl a2 = prodc(vec(kaphat))^((k-3)/2) * (kaphat[0]-kaphat[1]); 			
	decl a3 = exp(-sumc(vec(kaphat))/2)*density(g_M,/*alpha*/2, k/2, kappa/4, kaphat);
	return a1*a2*a3; 
}

class NRP
{
	decl m_kappa, m_kap1hat, m_fchi2;		//fchi2 is a flag: if true, use chi2 cvs, otherwise conditional
	NRP(const fChi2);
	Setkappa1(const kappa1);
	toIntwrt2(const x2);
	toIntwrt1(const x1);
};
NRP::NRP(const fChi2)
{
	m_fchi2 = fChi2;
}
NRP::Setkappa1(const kappa1)
{
	m_kappa = kappa1|0;
}
NRP::toIntwrt2(const x2)
{
	decl pdf = jointdensEvals(g_k, m_kappa, m_kap1hat~x2);
	return pdf;// > 0 ? pdf : 0;
}
NRP::toIntwrt1(const x1)
{
	decl result, sterr;
	decl kap2hatbound;
	if(m_fchi2)
		kap2hatbound = min(g_kap2hatbound,x1);	// use constant cv (if less than x1)
	else{
		decl idx = sumc(g_quants[][0].<x1);		// index of ceiling in x values.
		decl x_lo, x_up;
		x_lo = g_quants[idx-1][0];
		x_up = g_quants[idx][0];
		decl y_up =  g_quants[idx][1], y_lo = g_quants[idx-1][1];
		kap2hatbound = interpolate(x1, x_up, x_lo, y_up, y_lo);	   // conditional critical value interpolated from table
	}
	m_kap1hat = x1;
	decl iq = QAG(toIntwrt2, 0, kap2hatbound, 1, &result, &sterr);
	if(iq){
		println("%c",{"kappa1","x1","c(x1)","int","sterr","q"},m_kappa[0]~x1~kap2hatbound~result~sterr~iq);
		iq = QAG(toIntwrt2, 0, kap2hatbound, 2, &result, &sterr);
		if(iq){
			iq = QAG(toIntwrt2, 0, kap2hatbound, 3, &result, &sterr);
			if(iq){
				iq = QAG(toIntwrt2, 0, kap2hatbound, 4, &result, &sterr);
				if(iq){
					iq = QAG(toIntwrt2, 0, kap2hatbound, 5, &result, &sterr);
					if(iq)
						iq = QAG(toIntwrt2, 0, kap2hatbound, 6, &result, &sterr);
				}
			}
		}
	}
	
	return result; 			
}


main()									
{

	decl time = timer();
	decl crep = 1e6;						// # of MC replications for MC integration
	decl vlevel = <0.05>;
	decl vk = <5>;				
	decl ckappa1 = 42; 
	decl lb=0, ub=log(101), stepsize=ub/(ckappa1-1); 		//lb, ub, stepsize: lower and upper bound and stepsize for grid on kappa1.
	decl vkappa1 = exp(range(lb, ub, stepsize))-1;			//Set grid for kappa1 under H0.
	// check density is a density, i.e., integrates to 1
	decl check = new NRP(TRUE), rescheck, sterr;
	g_kap2hatbound = .Inf; g_k = vk[0];	check.Setkappa1(0);
	QPEPS(0, 1e-5);				// sets desired accuracy
	QAGI(check.toIntwrt1, 0.0001, 1, &rescheck, &sterr);
	println("Density integrates to ", rescheck);
	delete check;
	fuzziness(1e-5);
	if(!isfeq(rescheck, 1)){
		println("Density is wrong, check constant of integration");
		return;
	}
	for(decl ik = 0; ik < sizerc(vk); ++ik){
		g_k = vk[ik];
		for(decl j = 0; j < sizerc(vlevel); ++j){
			decl level = vlevel[j];						// nominal significance level
			decl mNullRejProb = constant(.NaN,ckappa1,4);	 							// stores NRP of cond AR test at different values of kappa1
			g_quants = loadmat(sprint("CVtables\\k_",g_k,"_level_",1-level,".in7"));   	// cv table
			g_kap2hatbound = quanchi(1-level, g_k-1);
			SetDrawWindow("Compare NI vs MC");
			DrawTitle(0, sprint("$k=",g_k,"$, $m_W=1$, $\\alpha = ", level,"$"));	  
			SetDraw(SET_LINE, 4, 1, 30);
			SetDraw(SET_LINE, 2, 0, 30);
			SetDraw(SET_LINE, 3, 2, 30);
			DrawXMatrix(0, constant(level,1,ckappa1),"",vkappa1,"", 0, 4);
			for(decl fchi2 = 0; fchi2 < 2; ++fchi2){
				decl model = new NRP(fchi2);
				parallel for(decl i = 0; i < ckappa1; ++i){
					decl modeli = clone(model);
					modeli.Setkappa1(vkappa1[i]);	
					decl result, sterr;
//					decl iq = QAG(modeli.toIntwrt1, 0.0001, 200, 6, &result, &sterr);		// takes longer, but can be more accurate in rare cases
					decl iq = QAGI(modeli.toIntwrt1, 0.0001, 1, &result, &sterr);
					mNullRejProb[i][fchi2] = (1-result);
				}
				delete model;
				println("\nlapsed time: ", timespan(time),"\n");
			}
			DrawXMatrix(0, mNullRejProb[][:1]',{"c:NI","GKMC:NI"},vkappa1,"", 0, <2;3>);
			parallel for(decl i = 0; i < ckappa1; ++i){
				decl mtestfunc = new matrix[crep][2];	// value of conditional and GKMC test functions for each draw (Nx2 matrix of 0s and 1s)
				ranseed(1);								// keep random numbers same across DGPs for smoother picture
				decl kappa1 = vkappa1[i];				// value of nuisance parameter for DGP j
				for(decl ix=0; ix < crep; ++ix)
				{
					decl x;
					decl mu = rann(g_k,2);
					decl mnc = zeros(g_k,2);				// initialize noncentrality (mean) of normals
					mnc[0][0] = sqrt(kappa1);			
					decl mX = mu+mnc;						// draw from non-central Wishart (via Normals)
					eigensym(mX'mX, &x);					// obtain its eigenvalues
					decl idx = sumc(g_quants[][0].<x[0]);	// index of ceiling in x1 values (to be used as conditional cv)
					decl x0, x_lo, x_up;
					x0 = x[0];
					x_lo = g_quants[idx-1][0];
					x_up = g_quants[idx][0];
					decl y_up =  g_quants[idx][1], y_lo = g_quants[idx-1][1];
					decl dcv = interpolate(x0, x_up, x_lo, y_up, y_lo);	   	 		// conditional critical value interpolated from table
					mtestfunc[ix][0] = x[1] .> dcv;									// conditional test function
					mtestfunc[ix][1] = x[1] .> quanchi(1-level,g_k-1);;				// GKMC test function
				}				
				mNullRejProb[i][2:] = meanc(mtestfunc);
			}
			DrawXMatrix(0, mNullRejProb[][2:]',{"c:MC","GKMC:MC"},vkappa1,"", 2, <2;3>);
			DrawText(0, "$\\kappa_1$", -1, -1, -1, -1, TEXT_XLABEL);
			DrawLegend(0, /*iOffsX*/500, /*iOffsY*/500, FALSE);		
			DrawAdjust(ADJ_AREAMATRIX, TRUE, FALSE, FALSE, 800);
			ShowDrawWindow();
			SaveDrawWindow(sprint("SizeCompNIvsMC.eps"));
		}
	}
	println("\nlapsed time: ", timespan(time),"\n");
	
}
					 	