/**	This file replicates power figures in the Appendix of GKM "A more powerfull subvector Anderson Rubin test in linear IV regression"
	Requires Powerbounds.in7 (produced by powerbounds.ox) and PowerCondTest.in7 (produced by PowerCondTest.ox).

	Need to vary f3D and fAMSEMW to get the different figures.
**/

#include <oxstd.oxh>
#include <oxdraw.oxh>
#import "Database"

main()
{
	decl time = timer();
	decl alpha = 0.05;								// significance level of test
	decl asVarsUp = {"AMS", "EMW"};					// variables holding upper bounds
	decl Varnames = {"Conditional", "GKMC"};		// variabls holding power curves
	decl bounds = new Database();
	bounds.Load("Powerbounds.in7");
	decl power = new Database();
	power.Load("PowerCondTest.in7");
	decl asPower = {"Upper bound", "ALFD", "$\\varphi_c$", "$\\varphi_{GKMC}$"};		// names for the 2-D plot
	decl sname;										// file name to save figure
	decl vk = <2;5;10;20>;
	decl vkappa2;
	decl vkap1mkap2 = <>;							// kap1mkap2 (kappa_1-kappa_2 under alternative)
	decl f3D = 0;									// if TRUE, plot 3D figures
	decl fAMSEMW = 1;								// f3D=0, does nothing, otherwise fAMSEMW = FALSE computes 3D power difference from ALFD power bound, fAMSEMW=1 from min(AMS,EMW) power bound
	for(decl ik = 0; ik < sizerc(vk); ++ik){
		decl k = vk[ik];			  					// which k to plot
		for(decl idplot = 0; idplot<8; ++idplot){		// which value of kappa1-kappa2 to plot for 2D
			if(f3D*idplot)
				continue;								// no point resaving 3D plot at different kappa1-kappa2
			vkap1mkap2 = <>;							// kappa_1-kappa_2 under alternative
			decl id = -1;								// initialize indix of kap1mkap2
			decl mPower2D, mPower3D;
			SetDraw(SET_LINE, 2, 2, 30);
			SetDraw(SET_LINE, 1, 0, 20);
			SetDraw(SET_LINE, 3, 1, 20);
			SetDraw(SET_LINE, 4, 3, 20);
			for(decl i=0; i<bounds.GetSize(); ++i)
			{
				if(k != bounds.GetVar("k")[i])
					continue;
				decl kap1mkap2 = bounds.GetVar("kappa1-kappa2")[i];	
				if(vkap1mkap2 == <> || kap1mkap2 != vkap1mkap2[id]){
					if(!f3D && id==idplot){								// plot 2D power curves in the same figure for kap1mkap2=0
						DrawTitle(id, sprint("k=", k, ", $\\kappa_1-\\kappa_2=$",vkap1mkap2[id]));
						DrawXMatrix(id, mPower2D, asPower, vkappa2, "", 0, 1);
						DrawText(id, "$\\kappa_2$", -1, -1,-1, -1, TEXT_XLABEL);
						DrawLegend(id, 500, 500, FALSE); 
						DrawAdjust(ADJ_AREA_Y, id, 0, 1);
						DrawAdjust(ADJ_LEGEND, id, 1, -1, -1, 0);
					}				
					vkappa2 = bounds.GetVar("kappa2")[i];		
					mPower2D = min(bounds.GetVar(asVarsUp)[i][])|bounds.GetVar("ALFD")[i]|power.GetVar(Varnames)[i][]';	
					vkap1mkap2 ~= kap1mkap2;
					++id;
				}
				else{
					vkappa2 ~= bounds.GetVar("kappa2")[i];
					mPower2D ~= (min(bounds.GetVar(asVarsUp)[i][])|bounds.GetVar("ALFD")[i]|power.GetVar(Varnames)[i][]');
				}
			}
			if(!f3D && idplot==sizerc(vkap1mkap2)-1){								// plot 2D power curves in the same figure for kap1mkap2=0
				DrawTitle(idplot, sprint("k=", k, ", $\\kappa_1-\\kappa_2=$",vkap1mkap2[id]));
				DrawXMatrix(idplot, mPower2D, asPower, vkappa2, "", 0, 1);
				DrawText(idplot, "$\\kappa_2$", -1, -1,-1, -1, TEXT_XLABEL);
				DrawLegend(idplot, 500, 500, FALSE); 
				DrawAdjust(ADJ_AREA_Y, idplot, 0, 1);
				DrawAdjust(ADJ_LEGEND, idplot, 1, -1, -1, 0);
				DrawAdjust(ADJ_PAPERSCALE, 150);
				DrawAdjust(ADJ_AREAMATRIX, -1,-1, 0, 760);
				sname = sprint("Power2D_k_",k);
			}				
			if(f3D){
				mPower3D = zeros(sizerc(vkap1mkap2), sizerc(vkappa2));
				for(decl i=0; i<bounds.GetSize(); ++i)
				{
					if(k != bounds.GetVar("k")[i])
						continue;
					decl kap1mkap2 = bounds.GetVar("kappa1-kappa2")[i];
					decl kappa2 = bounds.GetVar("kappa2")[i];
					decl vpower = fAMSEMW ? bounds.GetVar({"AMS","EMW"})[i][]~power.GetVar({"Conditional"})[i]
								: bounds.GetVar({"ALFD"})[i]~power.GetVar({"Conditional"})[i];
					mPower3D[vecindex(vkap1mkap2 .== kap1mkap2)][vecindex(vkappa2 .== kappa2)] = vpower[1+fAMSEMW]-min(vpower[range(0,fAMSEMW)]);	// Power of conditional test - ALFD or min(AMS,EMW) power bound
				}										  
				DrawTitle(ik, sprint("k = ",k));
				DrawXYZ(ik, vkappa2, vkap1mkap2, mPower3D', 0, "$\\kappa_2$", "$\\kappa_1-\\kappa_2$", "");
				DrawText(ik, "$\\kappa_2$", -1, -1,-1, -1, TEXT_XLABEL); 
				DrawText(ik, "$\\kappa_1-\\kappa_2$", -1, -1,-1, -1, TEXT_YLABEL); 
				sname = !fAMSEMW ? "Power3D_ALFD" : "Power3D_AMSEMW";
			}
		}
		if(!f3D){
			ShowDrawWindow();
			SaveDrawWindow(sprint(sname,".eps"));
		}
	}
	if(f3D){
		ShowDrawWindow();
		SaveDrawWindow(sprint(sname,".eps"));
	}
	delete bounds;
	delete power;
}