/** Saves the critical value tables reported in the Appendix of GKM "A more powerfull subvector Anderson Rubin test in linear IV regression"
	in TeX format.
	The file requires that Critical_Value_Tables.ox be run beforehand and cv tables be stored in .in7 format
**/

#include <oxstd.oxh>
#include <oxfloat.oxh>
#include <oxdraw.oxh>

TexTable(const f, const k, const valpha, const asnames, const amstuff)
/**	returns Tabular environment in tex **/
{
	fprintln(f, "\\begin{table}[tbp] \\centering");
	decl cp = sizeof(asnames);
	decl sc = "";
	for(decl i = 0; i < cp; ++i)
			sc ~= (i == cp-1) || !imod(i, 2) ? "r" : "r|";
	fprintln(f,"\\begin{tabular}{", sc, "}");
	fprintln(f,"\\hline \\hline");
	fprintln(f,"\\multicolumn{",cp,"}{c}{", sprint("$k-m_W=", k-1,"$"),"}\\\\");
	fprintln(f,"\\hline");			  
	for(decl a = 0; a < sizeof(amstuff); ++a){
		fprintln(f,"\\multicolumn{",cp,"}{l}{", sprint("$\\alpha = ", "%.0f", 100*valpha[a]),"\%$}\\\\");
		fprintln(f,"\\hline");			  
		decl i, sp;
	//	Get column headings
		sp = asnames[0];
		for(i = 1; i < cp; ++i)
			sp += sprint(" & ", asnames[i]);
		fprintln(f,sp,"\\\\");
		fprintln(f,"\\hline");			  
	
	//	Each row separately
		decl cn = sizeof(amstuff[a]);
		for(decl j = 0; j < cn; ++j)
		{
			sp = sprint("%.1f", amstuff[a][j][0]);
			for(i = 1; i < cp; ++i)
			{
				if(isstring(amstuff[a][j][i]))
					sp += sprint(" & ", amstuff[a][j][i]);
				else if(isnan(amstuff[a][j][i]))
					sp += sprint(" & ");
				else if(amstuff[a][j][i] == .Inf)
					sp += sprint(" & $\\infty$");
				else
					sp += sprint(" & ",	imod(i,2) ? (amstuff[a][j][i-1] >= 1000 ? "%.3f" : "%.1f")
					: amstuff[a][j][i]<1000 ? "%.1f": "%.0f", amstuff[a][j][i]);
	
			}
			fprintln(f,sp,"\\\\");
		}
		fprintln(f,"\\hline");			  
		if(a < sizeof(amstuff)-1)
			fprintln(f,"\\\\");
	}
	fprintln(f,"\\hline \\end{tabular}");
	fprintln(f, "\\caption{$1-\\alpha$ quantile of the conditional distribution,  with density given in (\\ref{eq: approx cond pdf}),", 
	"cv=$c_{1-\\alpha}\left(\hat{\\kappa}_1,k-m_W\\right)$ at different values of the conditioning variable $\hat{\kappa}_1$. Computed by numerical integration.}");
	fprintln(f, "\\label{t: conditional cvs for k-mw=", k-1,"}");
	fprintln(f, "\\end{table}");
	fprintln(f, "\\clearpage");
}

main()
{
	decl time = timer();
	decl v_level = <0.9,0.95,0.99>;		// confidence level (1- significance level)

	decl sfile = sprint("CVtables.tex");
	//Write to tex file: 
	decl f = fopen(sfile, "wt"); 	//T
	fprintln(f,"\\documentclass{article}");
	fprintln(f,"\\usepackage{geometry}[1in]");
	fprintln(f,"\\begin{document}");
	for(decl k = 2; k <= 21; ++k)
	{
		decl quantiles, vkap1hat;
		decl amstuff = new array[sizerc(v_level)];
		decl asnames;
		for(decl i=0; i<sizec(v_level); ++i)
		{
			decl data = loadmat(sprint("CVtables\\k_",k,"_level_",v_level[i],".in7"));
			vkap1hat = data[1:][0];
			quantiles = data[1:][1];

			decl cdata = sizer(quantiles);
			
			decl n = k < 11 ? 7 : 7;				// choose number of columns in table to fit nicely on page
			decl kap1tab = shape(vkap1hat,ceil(cdata/n), n);
			decl quantab = shape(quantiles,ceil(cdata/n),n);
			decl fortab = new matrix[sizeof(kap1tab)][sizec(kap1tab)*2];
			asnames = new array[sizec(fortab)];
			for(decl t = 0; t < sizec(kap1tab); ++t){
				fortab[][2*t:2*t+1] = kap1tab[][t]~quantab[][t];
				asnames[2*t:2*t+1] = {"$\\hat{\\kappa}_1$","cv"};
			}
			fortab = isdotfeq(fortab, 0) .? .NaN .: fortab;
			amstuff[i] = fortab;
		}
		TexTable(f, k, 1-v_level, asnames, amstuff);
	}
	fprintln(f,"\\end{document}");
	fclose(f);
	println("\nlapsed time: ", timespan(time),"\n");
}