/**	This file replicates 3D figures of the size of the AMS and EMW bounds in the Appendix of GKM "A more powerfull subvector Anderson Rubin test in linear IV regression"
	Requires Powerbounds.in7 (produced by powerbounds.ox)
**/

#include <oxstd.oxh>
#include <oxdraw.oxh>
#import "Database"

main()
{
	decl time = timer();
	decl data = new Database();
	data.Load("Powerbounds.in7");
	decl sVar = "sizeAMS";	// use "sizeAMS" for the AMS test and "sizeEMW" for the EMW test
	decl vk = <2;5;10;20>;
	for(decl ik = 0; ik < 4; ++ik){
		decl k = vk[ik];			  				// which k to plot
		decl vkappa2;
		decl vkap1mkap2 = <>;						// kappa_1-kappa_2 under alternative
		for(decl id = 0; id<8; ++id){				
			if(id)
				continue;							// no point resaving 3D plot at different kappa1-kappa2
			decl id = -1;							// initialize index of kappa1-kappa2
			for(decl i=0; i<data.GetSize(); ++i)
			{
				if(k != data.GetVar("k")[i])
					continue;
				decl kap1mkap2 = data.GetVar("kappa1-kappa2")[i];	
				if(vkap1mkap2 == <> || kap1mkap2 != vkap1mkap2[id]){
					vkappa2 = data.GetVar("kappa2")[i];		// restart vkappa2
					vkap1mkap2 ~= kap1mkap2;
					++id;
				}
				else
					vkappa2 ~= data.GetVar("kappa2")[i];
			}
		}
		decl mSize = zeros(sizerc(vkap1mkap2), sizerc(vkappa2));
		for(decl i=0; i<data.GetSize(); ++i)
		{
			if(k != data.GetVar("k")[i])
				continue;
			decl kap1mkap2 = data.GetVar("kappa1-kappa2")[i];
			decl kappa2 = data.GetVar("kappa2")[i];
			mSize[vecindex(vkap1mkap2 .== kap1mkap2)][vecindex(vkappa2 .== kappa2)] = data.GetVar(sVar)[i][];		
		}
		DrawTitle(ik, sprint("k=",k));
		DrawXYZ(ik, vkappa2, vkap1mkap2, mSize', 0, "$\\kappa_2$", "$\\kappa_1-\\kappa_2$", "");
		DrawText(ik, "$\\kappa_2$", -1, -1,-1, -1, TEXT_XLABEL); 
		DrawText(ik, "$\\kappa_1-\\kappa_2$", -1, -1,-1, -1, TEXT_YLABEL); 
	}
	ShowDrawWindow();
	SaveDrawWindow(sprint(sVar,".eps"));
}