/**	This file reproduces Table 1 in	GKM "A more powerfull subvector Anderson Rubin test in linear IV regression"
**/

#include <oxstd.oxh>
#include <oxfloat.oxh>
#include <quadpack.oxh>
#import <solvenle>
#include <conddens.ox>

steps(const start, const dmax, const cmaxsteps)
{
	decl stepsize = 0.1;
	for(decl s = 0; s < 10; ++s){
		if(dmax/stepsize<cmaxsteps-1)
			break;
		stepsize += 0.1;
	}

	decl vgrid = range(start, dmax, stepsize);
	if(!isfeq(vgrid[sizec(vgrid)-1],trunc(dmax*10)/10))
		vgrid ~= trunc(dmax*10)/10;
	return vgrid;
}
TexTable(const f, const asnames, const mstuff)
/**	returns Tabular environment in tex **/
{
	//f: filename
	//asnames: column heading
	//mstuff: data
	decl cp = sizeof(asnames);
	decl sc = "";
	for(decl i = 0; i < cp; ++i)
			sc ~= (i == cp-1) || !imod(i, 2) ? "r" : "r|";
	fprintln(f,"\\begin{tabular}{", sc, "}");
	fprintln(f,"\\hline \\hline");
	decl i, sp;
//	Get column headings
	sp = asnames[0];
	for(i = 1; i < cp; ++i)
		sp += sprint(" & ", asnames[i]);
	fprintln(f,sp,"\\\\");
	fprintln(f,"\\hline");			  

//	Each row separately
	decl cn = sizeof(mstuff);
	for(decl j = 0; j < cn; ++j)
	{
		sp = sprint("%.1f", mstuff[j][0]);
		for(i = 1; i < cp; ++i)
		{
			if(isstring(mstuff[j][i]))
				sp += sprint(" & ", mstuff[j][i]);
			else if(isnan(mstuff[j][i]))
				sp += sprint(" & ");
			else if(mstuff[j][i] == .Inf)
				sp += sprint(" & $\\infty$");
			else
				sp += sprint(" & ",	imod(i,2) ? (mstuff[j][i-1] >= 1000 ? "%.2f" : "%.1f")
				: mstuff[j][i]<100 ? "%.1f": "%.0f", mstuff[j][i]);

		}
		fprintln(f,sp,"\\\\");
	}
	fprintln(f,"\\hline \\hline \\end{tabular}");
}

main()
{
	decl time = timer();
	decl level = 0.05;							// significance level
	decl data = 0;//loadmat(sprint("CVtables\\k_5_level_",1-level,".in7"));	 // load table of cond cvs: two columns, kap1hat ~ cv
	QPEPS(0, 1e-3);				// sets desired accuracy
	decl quantiles, vkap1hat;
	if(any(data)){
		quantiles = data[1:][1]'; vkap1hat = data[1:][0]';
	}
	else									// if the table has not been computed and store, you need to compute it
	{
		g_dquant = 1-level;
		g_k = 5;
		decl dqchi2 = quanchi(g_dquant,g_k-1);		// quantile of chi2
		decl vgrid = range(0.1,300,.1);
		quantiles = vkap1hat = <>;						
		decl vqgrid;
		decl cmaxsteps = 80;
		decl q;		
		for(decl j = 0; j < sizerc(vgrid); ++j)
		{
			g_kap1hat = vgrid[j];
			q = min(dqchi2,g_dquant*g_kap1hat);		// initialize the quantile
			decl ires = SolveNLE(distfunc, &q, /*iMode*/-1, /*funcJac*/<>, /*dEps1*/-1, /*dEps2*/-1,/*mxIter*/-1, /*iPrint*/0);
			q = ceil(10*q)/10;								// round upwards to 1 decimal
			if(sizec(quantiles)==1)
				vqgrid = steps(quantiles[0], dqchi2, cmaxsteps);
			if(isfeq(q, g_kap1hat))							// skip all entries where q(x) = x, where x = kap1hat
				continue;
			if(sizec(quantiles)>1 && !any(isdotfeq(q,vqgrid)))		// skip all entries where q(x_i) is not in the grid of quantiles
				continue;
			if(sizec(quantiles) && isfeq(q,quantiles[sizec(quantiles)-1]))		// also skip all entries where q(x_i) = q(x_i-1)
				continue;
			if(q > dqchi2)									// if the rounded q exceeds the (unrounded) chi2, stop
				break;								
			quantiles ~= q;
			vkap1hat ~= vgrid[j];
			if(isfeq(q,round(10*dqchi2)/10))				// stop when the rounded q is within one decimal from chi2
				break;
		}
		g_kap1hat = 1000;							// add one final case to approximate chi2 quantile
		q = dqchi2;									// initialize at last chi2 quantile
		SolveNLE(distfunc, &q, /*iMode*/-1, /*funcJac*/<>, /*dEps1*/-1, /*dEps2*/-1,/*mxIter*/-1, /*iPrint*/0);
		quantiles ~= q;
		vkap1hat ~=	g_kap1hat;
	}		
	decl sfile = sprint("Table1.tex");
	//Write to tex file: 
	decl f = fopen(sfile, "wt"); 
	fprintln(f,"\\documentclass{article}");
	fprintln(f,"\\usepackage{geometry}[1in]");
	fprintln(f,"\\begin{document}");
	decl cdata = sizec(quantiles);
	// plot in double columns:
	decl ccols = 9;									// # of (double) columns in table for paper
	decl kap1tab = shape(vkap1hat,ceil(cdata/ccols), ccols);
	decl quantab = shape(quantiles,ceil(cdata/ccols),ccols);
	decl fortab = new matrix[sizeof(kap1tab)][sizec(kap1tab)*2];
	decl asnames = new array[sizec(fortab)];
	for(decl t = 0; t < sizec(kap1tab); ++t){
		fortab[][2*t:2*t+1] = kap1tab[][t]~quantab[][t];
		asnames[2*t:2*t+1] = {"$\\hat{\\kappa}_1$","cv"};
	}
	fortab = isdotfeq(fortab, 0) .? .NaN .: fortab;
	
	TexTable(f, asnames, fortab);

	fprintln(f,"\\end{document}");
	fclose(f);


	println("\nlapsed time: ", timespan(time),"\n");
}