/** This file reproduces Table 22 (size of conditional test) in GKM "A more powerfull subvector Anderson Rubin test in linear IV regression"
	Must be run after the file "Critical_Value_Tables.ox"
	Takes about 75 minutes on a 2.2GHz 44-core multi-threaded Windows server
**/

#include <oxstd.oxh>
#include <oxfloat.oxh>
#include <oxdraw.oxh>
#include <oxprob.oxh>
#include <interpolate.ox>		

TexTable(const f, const asnames, const amstuff)
/**	returns Tabular environment in tex **/
{
	fprintln(f, "\\begin{table}[tbp] \\centering");
	decl cp = sizeof(asnames);
	decl sc = "";
	for(decl i = 0; i < cp; ++i)
			sc ~= (i == cp-1) ? "r" : "r|";
	for(decl a = 0; a < sizeof(amstuff); ++a){
		fprintln(f,"\\begin{tabular}{", sc, "}");
		fprintln(f,"\\hline \\hline");
		fprintln(f," & \\multicolumn{",cp-1,"}{c}{$\\alpha$}\\\\");
		fprintln(f,"\\hline");			  
		for(decl i = 0; i < cp; ++i)
			fprint(f, asnames[i], i < cp - 1 ? " & " : "\\\\");
		fprintln(f," \n\\hline");			  
		
	//	Each row separately
		decl cn = sizeof(amstuff[a]);
		for(decl j = 0; j < cn; ++j){
			for(decl i = 0; i < cp; ++i)
				fprint(f, i == 0 ? "%.0f" : "%.4f", amstuff[a][j][i], i == cp-1 ? "\\\\ \n" : " & ");
		}
		fprintln(f,"\\hline \\hline \\end{tabular}");
	}
	fprintln(f, "\\caption{Size of $\\alpha$ level conditional subvector AR test for different $k$ with $m_W=1$,", 
	"using critical values given in Tables \\ref{t: conditional cvs for k-mw=1} to \\ref{t: conditional cvs for k-mw=20} and linear interpolation.", 
	"Computed using $10^6$ Monte Carlo replications.}");
	fprintln(f, "\\label{t: size}");
	fprintln(f, "\\end{table}");
}

main()
{
	decl time = timer();
	decl vlevel = <0.1,0.05,0.01>;		// significance level alpha
	decl ckappa1 = 42; 
	decl lb=0, ub=log(101);
	decl stepsize=ub/(ckappa1-1); 		//lb, ub, stepsize: lower and upper bound and stepsize for grid on kappa1.
	decl vkappa1 = exp(range(lb, ub, stepsize))-1;			//Set grid for kappa1 under H0.
	decl mNRF = zeros(ckappa1, 2);
	decl vmaxrej = vlevel;
	decl crep = 1e6;									// # of MC simulations
	decl msize = <>;									// to store results on size
	for(decl k = 2; k <= 21; ++k){
		for(decl i = 0; i < sizerc(vlevel); ++i){
			decl data = loadmat(sprint("CVtables\\k_",k,"_level_",1-vlevel[i],".in7"));	 // load table of cond cvs: two columns, kap1hat ~ cv
			decl level = vlevel[i];
			parallel for(decl j = 0; j < ckappa1; ++j)	
			{
				decl mtestfunc = new matrix[crep][1];	// value of conditional test function for each draw (Nx1 matrix of 0s and 1s)
				ranseed(1);								// keep random numbers same across DGPs for smoother picture
				decl kappa1 = vkappa1[j];				// value of nuisance parameter for DGP j
				for(decl ix=0; ix < crep; ++ix)
				{
					decl x;
					decl mu = rann(k,2);
					decl mnc = zeros(k,2);				// initialize noncentrality (mean) of normals
					mnc[0][0] = sqrt(kappa1);			
					decl mX = mu+mnc;					// draw from non-central Wishart (via Normals)
					eigensym(mX'mX, &x);				// obtain its eigenvalues
					decl idx = sumc(data[][0].<x[0]);		// index of ceiling in x1 values (to be used as conditional cv)
					decl x0, x_lo, x_up;
					x0 = x[0];
					x_lo = data[idx-1][0];
					x_up = data[idx][0];
					decl y_up =  data[idx][1], y_lo = data[idx-1][1];
					decl dcv = interpolate(x0, x_up, x_lo, y_up, y_lo);			   // conditional critical value interpolated from table
										
					mtestfunc[ix][0] = x[1] .> dcv;				// conditional test function
				}				
				mNRF[j][] = meanc(mtestfunc);
			}	
			vmaxrej[i] = max(mNRF[][0], level);
		}
		println("k = ", k, "%c", {"level", "maxNRP"}, vlevel'~vmaxrej');		// uncomment to get intermediate output
		msize |= k~vmaxrej;
	}
	decl asnames = {"k"};
	for(decl i = 0; i < sizerc(vlevel); ++i)
		asnames ~= sprint(vlevel[i]);
	savemat("size.csv", msize, asnames);
//	msize = loadmat("size.csv",  &asnames);				// if you want to save the tex table from already stored results, uncomment this line and comment out the previous outer for loop
	decl sfile = sprint("size.tex");
	//Write to tex file: 
	decl f = fopen(sfile, "wt"); 	
	fprintln(f,"\\documentclass{article}");
	fprintln(f,"\\begin{document}");
	decl n = sizeof(msize);
	TexTable(f, asnames, {msize[:n/2-1][],msize[n/2:][]});
	fprintln(f,"\\end{document}");
	fclose(f);
	
	println("\nlapsed time: ", timespan(time),"\n");

}