/**	This file contains the conditional density function in eq. 2.12 in GKM "A more powerfull subvector Anderson Rubin test in linear IV regression"
**/

#include <oxstd.oxh>

// global variables
decl g_kap1hat;						// value of conditioning statistic, largest eigenvalue of Wishart matrix
decl g_k;							// # of instuments
decl g_b = 1e6;						// constant to approximate the 1F1 function from 2F1 (must be very large)
decl g_kap1;	   					// true nonzero eigenvalue of noncentrality matrix
decl g_dquant;						// which quantile of the conditional distribution, e.g., 0.9 for 90th percentile

dens2g1(const x)
/** Computes the density function of kap2hat | kap1hat
	NOTE: support of 0 <= x <= kap1hat
**/
{
	if(x>g_kap1hat)
		return 0;
	if(x<0)
		return 0;
//	else		
	decl d1F1;						// holds the 1F1 function that appears in the density
	d1F1 = hyper_2F1((g_k-1)/2, g_b, (g_k+2)/2, -g_kap1hat/2/g_b);
	if(g_kap1hat > 1000 && d1F1 == .Inf)		   		// when the argument is large, numerical evaluation of 1F1 can fail (and it does!)
		d1F1 = exp(g_kap1hat)/sqrt(2*M_PI*g_kap1hat);   // in those cases, use the asymptotic expression lim_x->\infty [1F1(a,b;x) = exp(x)/sqrt(2*M_PI*x) (Olver, 1974, p. 435)
	return gammafact((g_k+2)/2) / gammafact((g_k-1)/2) * 2 .* exp(-x/2)
		*g_kap1hat^(-g_k/2) .* x.^((g_k-3)/2) .* sqrt(g_kap1hat-x)/sqrt(M_PI)
		/d1F1;
}	

distfunc(const avF, const x)
{
	decl result, sterr;
	decl iq = QAG(dens2g1, 0, x, 6, &result, &sterr);
	avF[0] = result - g_dquant;
	return !iq;
}

